/****************************************************************************
 * drivers/wireless/generic/si4463_modemcalc.h
 *
 *   Copyright (C) 2018 Sebastien Lorquet. All rights reserved.
 *   Author: Sebastien Lorquet <sebastien@lorquet.fr>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name NuttX nor the names of its contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#ifdef SI4463_MODEMCALC_H
#define SI4463_MODEMCALC_H

/* Parameters used by modem calculator to define register values.
 * Units are slightly modified from official calculator to improve precision.
 */

struct si4463_modemparams_s
{
  uint32_t freq_xo;          /* xtal frequency in Hz [30M] */
  uint32_t crystal_tol;      /* xtal tolerance in ppm [20] */
  uint8_t  if_mode;          /* 0=Scale, 1=Zero, 2=Fixed [2] */
  bool     High_perf_Ch_Fil; /* 0=Normal, 1=HighPerf [1] */
  int      OSRtune;          /* OversamplingRateTune range -20..20 [0] */
  int      Ch_Fil_Bw_AFC;    /* bw filter in afc tracking mode 0=Rx1, 1=Rx2 [0] */
  bool     ant_div;          /* bool antenna diversity [false] */
  int      pm_pattern;       /* see below [0] */
  uint8_t  modulation_type;  /* 0=CW, 1=OOK, 2=2FSK, 3=2GFSK, 4=4FSK, 5=4GFSK [3] */
  uint32_t Rsymb;            /* data rate in mBps (!!) [50k x 1000] */
  uint32_t Fdev;             /* frequency deviation in mHz (!!) [25k x 1000] */
  uint32_t RXBW;             /* RX bandwidth in Hz (OOK_Bandwidth) [200k] */
  bool     Manchester;       /* bool enable [false] */
  bool     afc_en;           /* int AutoFrequencyControl [0] */
  int      Max_Rb_Error;     /* 0=1 % 1=10 % 15=15 % [0] */
  int      Chip_Version;     /* 0=RevA_VCOv0, 1=RevA_VCOv1, 2=RevB_VCOv0, 3=RevB_VCOv1 [2] */
  uint32_t fc;               /* main frequency in Hz [915M] */
  int      TC;               /* RampingTime [29] */
  uint32_t fhst;             /* FrequencyHoppingChannelSpacing in mHz (!!) [25k x 1000] */
  bool     inputBW;          /* bool InputRxBandwidth [false] */
  bool     BER_mode;         /* bool BitErrorRateTest [false] */
  bool     raw_demod;        /* bool RawDataOutput [false] */
  int      dsource;          /* 0=PH, 1=Direct, 2=AsyncDirect, 3=MCU [0] */
  bool     hi_pfm_div_mode;  /* bool HighPerformanceDividerMode [true] */
};

struct si4463_propertyset_s
{
  uint16_t first;
  uint8_t count;
  uint8_t props[16];
};

/****************************************************************************
 * Name: si4463_modemcalc
 *
 * Description:
 *   Compute chip register contents based on RF parameters.
 *   Output modem properties in a data array.
 *   Returns the number of property sets used to define modem parameters.
 *
 ****************************************************************************/

int si4463_modemcalc(FAR struct si4463_modemcalc_s *requested,
                     FAR struct si4463_modemcalc_s *actual,
                     FAR struct si4463_propertyset_s *props, int maxprops);

#endif
